@@ -1,7 +1,7 @@
 /* Copyright © 2007-2015 Jakub Wilk <jwilk@jwilk.net>
  * Copyright © 2009 Mateusz Turcza
  *
- * This file is part of pdfdjvu.
+ * This file is part of pdf2djvu.
  *
  * pdf2djvu is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -39,6 +39,8 @@
 #include "debug.hh"
 #include "i18n.hh"
 #include "pdf-unicode.hh"
+#include "string-printf.hh"
+#include "sys-time.hh"
 #include "system.hh"
 
 
@@ -289,19 +291,32 @@ pdf::Timestamp::Timestamp()
 
 pdf::Timestamp pdf::Timestamp::now()
 {
-  time_t unix_now;
   pdf::Timestamp result;
   result.dummy = false;
   result.tz_sign = '+';
   result.tz_hour = 0;
   result.tz_minute = 0;
-  time(&unix_now);
+  time_t unix_now = time(NULL);
   if (unix_now == static_cast<time_t>(-1))
-    throw pdf::Timestamp::Invalid();
-  struct tm *c_timestamp = gmtime(&unix_now);
-  if (c_timestamp == NULL)
-    throw pdf::Timestamp::Invalid();
-  result.timestamp = *c_timestamp;
+    throw_posix_error("time()");
+  struct tm *local_tm = localtime(&unix_now);
+  if (local_tm == NULL)
+    throw_posix_error("localtime()");
+  time_t unix_now_l = timegm(local_tm);
+  if (unix_now_l == static_cast<time_t>(-1))
+    throw_posix_error("timegm()");
+  time_t tz_offset = unix_now_l - unix_now;
+  if (tz_offset >= 0)
+    result.tz_sign = '+';
+  else
+  {
+    result.tz_sign = '-';
+    tz_offset = -tz_offset;
+  }
+  tz_offset /= 60;
+  result.tz_hour = tz_offset / 60;
+  result.tz_minute = tz_offset % 60;
+  result.timestamp = *local_tm;
   return result;
 }
 
