@@ -22,10 +22,7 @@
 #include <vector>
 
 #include "i18n.hh"
-
-#if __cplusplus < 201103
-#define decltype typeof
-#endif
+#include "string-printf.hh"
 
 namespace string_format
 {
@@ -50,8 +47,7 @@ namespace string_format
   {
   protected:
     std::string variable;
-    bool negative_offset;
-    unsigned int offset;
+    int offset;
     unsigned int width;
     bool auto_width;
     bool pad_0;
@@ -62,18 +58,42 @@ namespace string_format
     virtual void format(const Bindings &, std::ostream &) const;
   };
 
-  class IntegerOverflow : public std::overflow_error
+  class ValueError : public std::domain_error
   {
   public:
-    IntegerOverflow()
-    : std::overflow_error(_("Integer overflow"))
+    ValueError(const std::string &what)
+    : std::domain_error(what)
+    { }
+  };
+
+  class ValueTypeError : public ValueError
+  {
+  public:
+    ValueTypeError(const std::string &what)
+    : ValueError(what)
+    { }
+  };
+
+  class FormatError : public std::runtime_error
+  {
+  public:
+    FormatError(const std::string &var, const std::string &what)
+    : std::runtime_error(string_printf(_("Unable to format field {%s}: %s"), var.c_str(), what.c_str()))
     { }
   };
 
 }
 
+string_format::Value string_format::Bindings::get(const std::string &value) const
+{
+  const_iterator it = this->find(value);
+  if (it == this->end())
+    throw ValueError(_("no such variable"));
+  return it->second;
+}
+
 string_format::VariableChunk::VariableChunk(const std::string &description)
-: negative_offset(false), offset(0), width(0), auto_width(false), pad_0(false)
+: offset(0), width(0), auto_width(false), pad_0(false)
 {
   enum
   {
@@ -85,6 +105,7 @@ string_format::VariableChunk::VariableChunk(const std::string &description)
     END
   } mode = NAME;
   std::string::const_iterator it = description.begin();
+  int offset_sign = 1;
   while (it != description.end())
   {
     switch (mode)
@@ -97,7 +118,7 @@ string_format::VariableChunk::VariableChunk(const std::string &description)
           mode = WIDTH_1;
         else
         {
-          this->negative_offset = *it == '-';
+          offset_sign = (*it == '+') ? +1 : -1;
           mode = OFFSET_1;
         }
       }
@@ -109,19 +130,19 @@ string_format::VariableChunk::VariableChunk(const std::string &description)
         mode = OFFSET_2;
       }
       else
-        throw ParseError();
+        throw string_format::ParseError();
       break;
     case OFFSET_2:
       if (*it >= '0' && *it <= '9')
       {
-        if (this->offset > (UINT_MAX - 9) / 10)
-          throw IntegerOverflow();
+        if (this->offset > (INT_MAX - 9) / 10)
+          throw string_format::FormatError(this->variable, _("integer overflow"));
         this->offset = this->offset * 10 + (*it - '0');
       }
       else if (*it == ':')
         mode = WIDTH_1;
       else
-        throw ParseError();
+        throw string_format::ParseError();
       break;
     case WIDTH_1:
       if (*it == '0')
@@ -129,14 +150,14 @@ string_format::VariableChunk::VariableChunk(const std::string &description)
       else if (*it >= '0' && *it <= '9')
         this->width = *it - '0';
       else
-        throw ParseError();
+        throw string_format::ParseError();
       mode = WIDTH_2;
       break;
     case WIDTH_2:
       if (*it >= '0' && *it <= '9')
       {
         if (this->width > (UINT_MAX - 9) / 10)
-          throw IntegerOverflow();
+          throw string_format::FormatError(this->variable, _("integer overflow"));
         this->width = this->width * 10 + (*it - '0');
       }
       else if (*it == '*')
@@ -145,41 +166,64 @@ string_format::VariableChunk::VariableChunk(const std::string &description)
         mode = END;
       }
       else
-        throw ParseError();
+        throw string_format::ParseError();
       break;
     case END:
-      throw ParseError();
+      throw string_format::ParseError();
       break;
     }
     it++;
   }
   if (mode == NAME)
     this->variable = description;
+  this->offset *= offset_sign;
 }
 
-static unsigned int shift(unsigned int value, unsigned int offset, bool negative_offset)
+unsigned int string_format::Value::as_int(int offset)
 {
-  if (negative_offset)
+  if (!this->is_int)
+    throw string_format::ValueTypeError(_("type error: expected number, not string"));
+  if (offset < 0)
   {
-    if (offset > value)
-      value = 0;
+    unsigned int uoffset = -offset;
+    if (uoffset > this->v_uint)
+      return 0;
     else
-      value -= offset;
+      return this->v_uint - uoffset;
   }
-  else if (value + offset >= value)
-    value += offset;
   else
-    throw string_format::IntegerOverflow();
-  return value;
+  {
+    unsigned int uoffset = offset;
+    if (this->v_uint + uoffset >= this->v_uint)
+      return this->v_uint + uoffset;
+    else
+      throw string_format::ValueError(_("integer overflow"));
+  }
+}
+
+const std::string & string_format::Value::as_string()
+{
+  if (this->is_int)
+    throw string_format::ValueTypeError(_("type error: expected string, not number"));
+  return this->v_string;
 }
 
 void string_format::VariableChunk::format(const Bindings &bindings, std::ostream &stream) const
+try
 {
-  unsigned int value = shift(bindings.get(this->variable), this->offset, this->negative_offset);
+  Value value = bindings.get(this->variable);
   unsigned int width = this->width;
   if (this->auto_width)
   {
-    unsigned int max_value = shift(bindings.get("max_" + this->variable), this->offset, this->negative_offset);
+    unsigned int max_value;
+    try
+    {
+      max_value = bindings.get("max_" + this->variable).as_int(this->offset);
+    }
+    catch (string_format::ValueError &exc)
+    {
+      throw FormatError(this->variable, _("unknown maximum width"));
+    }
     unsigned int max_digits = 0;
     while (max_value > 0)
     {
@@ -191,8 +235,21 @@ void string_format::VariableChunk::format(const Bindings &bindings, std::ostream
   }
   stream
     << std::setfill(this->pad_0 ? '0' : ' ')
-    << std::setw(width)
-    << value;
+    << std::setw(width);
+  try
+  {
+    stream << value.as_int(this->offset);
+  }
+  catch (string_format::ValueTypeError)
+  {
+    if (this->offset != 0)
+      throw;
+    stream << value.as_string();
+  }
+}
+catch (string_format::ValueError &exc)
+{
+  throw FormatError(this->variable, exc.what());
 }
 
 string_format::Template::Template(const std::string &source)
@@ -229,7 +286,7 @@ string_format::Template::Template(const std::string &source)
         mode = TEXT;
       }
       else
-        throw ParseError();
+        throw string_format::ParseError();
       break;
     case FORMAT_1:
       if (*right == '{')
@@ -253,7 +310,7 @@ string_format::Template::Template(const std::string &source)
     right++;
   }
   if (mode != TEXT)
-    throw ParseError();
+    throw string_format::ParseError();
   if (left != source.end())
   {
     std::string text = source.substr(left - source.begin(), std::string::npos);
