@@ -1,7 +1,7 @@
 /* Copyright © 2007-2015 Jakub Wilk <jwilk@jwilk.net>
  * Copyright © 2009 Mateusz Turcza
  *
- * This file is part of pdfdjvu.
+ * This file is part of pdf2djvu.
  *
  * pdf2djvu is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -13,17 +13,17 @@
  * General Public License for more details.
  */
 
+#include "autoconf.hh"
 #include "system.hh"
 
 #include <algorithm>
 #include <cassert>
 #include <cerrno>
-#include <cstdarg>
+#include <climits>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <iostream>
-#include <limits>
 #include <sstream>
 #include <stdexcept>
 
@@ -38,13 +38,9 @@
 #endif
 
 #include "array.hh"
-#include "autoconf.hh"
 #include "debug.hh"
 #include "i18n.hh"
-
-#if USE_MINGW_ANSI_STDIO
-#define vsnprintf __mingw_vsnprintf
-#endif
+#include "string-printf.hh"
 
 /* constants
  * =========
@@ -370,10 +366,13 @@ File::File(const Directory& directory, const std::string &name)
   this->open(stream.str().c_str());
 }
 
-size_t File::size()
+File::streamoff File::size()
 {
+  File::streampos orig_pos = this->tellg();
   this->seekg(0, std::ios::end);
-  return this->tellg();
+  File::streamoff result = this->tellg();
+  this->seekg(orig_pos);
+  return result;
 }
 
 void File::reopen(bool truncate)
@@ -478,7 +477,7 @@ Cwd::Cwd(const std::string &path)
     rc = getcwd(buffer, size) == NULL;
     if (rc != 0)
     {
-      if (errno == ERANGE && size < std::numeric_limits<size_t>::max() / 2)
+      if (errno == ERANGE && size < SIZE_MAX / 2)
       {
         size *= 2;
         continue;
@@ -729,43 +728,6 @@ bool is_same_file(const std::string &path1, const std::string &path2)
 #endif
 }
 
-#if defined(va_copy)
-#elif defined(__va_copy)
-#define va_copy __va_copy
-#else
-#define va_copy(dest, src) memcpy((dest), (src), sizeof (va_list))
-#endif
-
-std::string string_vprintf(const char *message, va_list args)
-{
-  va_list args_copy;
-  va_copy(args_copy, args);
-  int length = vsnprintf(NULL, 0, message, args_copy);
-  va_end(args_copy);
-  if (length < 0)
-    throw_posix_error("vsnprintf");
-  if (length == std::numeric_limits<int>::max())
-  {
-    errno = ENOMEM;
-    throw_posix_error("vsnprintf");
-  }
-  Array<char> buffer(length + 1);
-  length = vsprintf(buffer, message, args);
-  if (length < 0)
-    throw_posix_error("vsprintf");
-  return static_cast<char*>(buffer);
-}
-
-std::string string_printf(const char *message, ...)
-{
-  va_list args;
-  va_start(args, message);
-  std::string result = string_vprintf(message, args);
-  va_end(args);
-  return result;
-}
-
-
 void prevent_pop_out(void)
 {
 #if WIN32
