@@ -2,7 +2,7 @@
 
 # Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
 #
-# This file is part of pdfdjvu.
+# This file is part of pdf2djvu.
 #
 # pdf2djvu is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -13,10 +13,14 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 
+import xml.etree.cElementTree as etree
+
 from tools import (
-    assert_regex,
-    assert_well_formed_xml,
+    assert_equal,
+    assert_not_equal,
+    assert_uuid_urn,
     case,
+    xml_find_text,
 )
 
 class test(case):
@@ -26,14 +30,21 @@ class test(case):
     def test_verbatim(self):
         self.pdf2djvu('--verbatim-metadata').assert_()
         xmp = self.extract_xmp()
-        assert_well_formed_xml(xmp)
-        assert_regex(xmp, '>application/pdf<')
+        xmp = etree.fromstring(xmp)
+        dcformat = xml_find_text(xmp, 'dc:format')
+        assert_equal(dcformat, 'application/pdf')
 
     def test_no_verbatim(self):
-        self.require_feature('GNOME XSLT')
+        self.require_feature('Exiv2')
         self.pdf2djvu().assert_()
         xmp = self.extract_xmp()
-        assert_well_formed_xml(xmp)
-        assert_regex(xmp, '>image/vnd.djvu<')
+        xmp = etree.fromstring(xmp)
+        dcformat = xml_find_text(xmp, 'dc:format')
+        assert_equal(dcformat, 'image/vnd.djvu')
+        instance_id = xml_find_text(xmp, 'xmpMM:InstanceID')
+        document_id = xml_find_text(xmp, 'xmpMM:DocumentID')
+        assert_not_equal(instance_id, document_id)
+        assert_uuid_urn(instance_id)
+        assert_uuid_urn(document_id)
 
 # vim:ts=4 sts=4 sw=4 et
