@@ -48,6 +48,13 @@ def noseimport(vmaj, vmin, name=None):
         return f
     return wrapper
 
+@noseimport(2, 7)
+def assert_greater(x, y):
+    assert_true(
+        x > y,
+        msg='{0!r} not greater than {1!r}'.format(x, y)
+    )
+
 @noseimport(2, 7)
 def assert_in(x, c):
     assert_true(
@@ -298,7 +305,10 @@ def count_ppm_colors(b):
     result = collections.defaultdict(int)
     for pixel in itertools.izip(ipixels, ipixels, ipixels):
         result[pixel] += 1
-    return result
+    return dict(
+        (''.join(key), value)
+        for key, value in result.iteritems()
+    )
 
 xml_ns = dict(
     dc='http://purl.org/dc/elements/1.1/',
